
header {
package com.ibm.ocl.parser;

import com.ibm.uml.*;
import com.ibm.ocl.types.*;
import com.ibm.ocl.types.impl.*;
import com.ibm.ocl.expressions.*;
import com.ibm.ocl.expressions.impl.OperationCallExpImpl;

import org.eclipse.emf.ecore.*;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.common.util.*;

import java.lang.String;
import java.util.Vector;
import java.util.HashSet;
import java.util.*;
}

class OclParser extends Parser;
		options { k = 2; defaultErrorHandler=false;  }		
  
{
	/*
	 * Factories for creating OCL AST nodes
	 */	 	
	TypesPackage typesPackage = TypesPackage.eINSTANCE;
    TypesFactory typesFactory = typesPackage.getTypesFactory();
    	
    ExpressionsPackage expressionsPackage = ExpressionsPackage.eINSTANCE;
    ExpressionsFactory expressionsFactory = expressionsPackage.getExpressionsFactory();
    	   		 
    UmlPackage umlPackage = UmlPackage.eINSTANCE;
    UmlFactory umlFactory = umlPackage.getUmlFactory();
    
 
    private boolean traceflag = true;	
    private VariableDeclaration self = null;
    	
    private boolean errorOccurred = false;
    private String errorString = "";		

    public void setTraceFlag(boolean flag) {
    	traceflag = flag;
    }
    
    public boolean getTraceFlag() {
    	return traceflag;
    }
    
    private void TRACE( String rule, String message) {
	    if (traceflag) {
	    	System.out.println(rule + ": " + message);
	    }
	}
		    
	private void TRACE(String rule, String message, Vector namelist) {		  			
		TRACE(rule, message + makeString(namelist)); 
	}
	
	private String makeString(Vector namelist) {
		String msg = "";
		for (int i = 0; i < namelist.size(); i++) {
			msg += " " + (String) namelist.elementAt(i);
		}
		return msg;
	}
	
	/**
	 * Convenience method for parsing OCL expressions
	 * String to be parsed is the argument for OclLexer.
	 * @return the parsed OCL expression
	 */
	public OclExpression parse() {
		boolean traceFlagSave = getTraceFlag();
		setTraceFlag(false);
		EList constraints = new BasicEList();
		try {
			classifierContextDeclCS(null, constraints);
		}
		catch (Exception e) {
			throw new IllegalArgumentException("The string cannot be parsed: " + e.getMessage());
		}
		com.ibm.uml.Constraint constraint = (com.ibm.uml.Constraint) constraints.get(0);	
		setTraceFlag(traceFlagSave);
		
		return constraint.getBody();	
	}
	
	/**
	 * Hub for all semantic parsing messages -- 
	 * Should be changed if want to continue parsing, and not terminate
	 * on first exception.
	 */
	public static void ERR(String message) throws SemanticException {
		throw new SemanticException(message);
	}
	    
	private void ERROR(String rule, String errorMessage) throws SemanticException {
	    String error = ("ERROR in " + rule + ": " + errorMessage);
	    errorOccurred = true;
	    
	    errorString += error;
	    ERR(error);
	    
	    /*if (traceflag) {
	    	System.out.println(error);
	    }*/
	    
	}
		
	private void ERROR(String rule, String errorMessage, Vector namelist)
	throws SemanticException {		  	
		String msg = errorMessage;
		for (int i = 0; i < namelist.size(); i++) {
			msg += " " + (String) namelist.elementAt(i);
		}
		ERROR(rule, msg); 
	}
	
	
	/**
	 * Generate a VariableDeclaration AST node, and add it to the environment.
	 * Variable declarations are generated for "self", let expression variables, and
	 * iterator and iterate variables, both implicit and explicit.
	 * For implicit variables, the name is generated by the Environment.
	 */
	private VariableDeclaration genVariableDeclaration(String rule, Environment env, String name, 
				EClassifier type, OclExpression initExp, boolean explicitFlag,
				boolean addToEnvironment) throws RecognitionException {
			
		VariableDeclaration vdcl = expressionsFactory.createVariableDeclaration();		
 		vdcl.setVarName(name);
 		vdcl.setType(type);
 		vdcl.setInitExpression(initExp);
 		
 		if (addToEnvironment) {
 			boolean result = env.addElement(name, vdcl, explicitFlag);
 			if (!result) {
 				if (name != null) {
 					ERROR(rule, "Variable already used: " + name);
 				} else {
 					ERROR(rule, "Variable declaration failed");
 				}
 			}
 		}
 		
 		if (addToEnvironment) {
 			TRACE(rule, "adding variable declaration for " + vdcl.getVarName());
 		}	
 		return vdcl;
	}
    	    
	/**
	 * Generate an OperationCallExp node. 
	 * operName is the input name of the operation, which
	 * must be matched against the datatype of the operation source.
	 */ 
	private OperationCallExp genOperationCallExp(Environment env, String rule, String operName, OclExpression source, EList args)
	throws RecognitionException {
	    	
	    OperationCallExp result;
	    boolean errorflag = false;	// flag to avoid propagating error messages
	    
	    result = expressionsFactory.createOperationCallExp();
	    result.setSource(source);	   
	    EClassifier sourceType = source.getType();
	    
	    // Performs method signature checking		
	    EOperation oper = EnvironmentImpl.lookupOperation(sourceType, operName, args);
	    if (oper == null) { 
		   	ERROR(rule, "Cannot find operation " + operName + " for the datatype");
		   	errorflag = true;
		   	result.setType( PrimitiveTypeImpl.OCL_VOID);
		} else {
		  	TRACE(rule, oper.getName());
	    	result.setReferredOperation(oper);
	    }
						    
		// Set up arguments
		EList callargs = result.getArguments();
		if (sourceType == PrimitiveTypeImpl.OCL_VOID) {
			errorflag = true;
		}
		if (args != null) {
		   	for (int i = 0; i < args.size(); i++) {
		   		OclExpression arg = (OclExpression) args.get(i);
		   		if (arg == null) {
		   			ERROR(rule, "Bad argument expression");
		   			errorflag = true;
		   			continue;
		   		}
		   		
		   		if (arg.getType() == PrimitiveTypeImpl.OCL_VOID) { 
		   			errorflag = true;
		   		}
		   		callargs.add(args.get(i));
		   	}
		}
				
		// Compute the result type, and perform conformance checking.
		if (oper != null) {
			EClassifier resultType = null;
			
		   	if (sourceType instanceof CollectionType) { 
		   		int opcode = ((CollectionTypeImpl) sourceType).getOperationCode(operName);
		   		result.setOperationCode(opcode);
		   		CollectionTypeImpl ct = (CollectionTypeImpl) sourceType;
				resultType = ct.getResultType(opcode, args);
		   	} else if (sourceType instanceof PrimitiveType) {
		   		int opcode = ((PrimitiveTypeImpl) sourceType).getOperationCode(operName);
		   		result.setOperationCode(opcode);
		   		PrimitiveTypeImpl pt = (PrimitiveTypeImpl) sourceType;
		   		resultType = pt.getResultType(opcode, args);
		   	} else {
		   		// source is a tuple, an EClass, or an enumeration
		   		int opcode = AnyTypeImpl.OCL_ANY_TYPE.getOperationCode(operName);
		   		result.setOperationCode(opcode);
		   		resultType = AnyTypeImpl.getResultType(sourceType, opcode, args);
		   	}
		   	if (resultType == null)
		   		resultType = EnvironmentImpl.getOclTypeFor(oper.getEType());	   		
		   	result.setType(resultType);
		}
			
		return result;
	}
}
           
    
/*
 * packageDeclarationCS
 */
packageDeclarationCS[EList constraints]
{ 	Vector pathname = null; 
	EPackage defaultPackage = null;
} :

	((	"package" pathname = pathNameCS 
		{ 
			defaultPackage = EnvironmentImpl.findPackage(pathname); 
		  	if (defaultPackage == null) {
		  	
		  		ERROR("packageDeclarationCS", "Unable to find package ", pathname); 
		 	 }
		  
		  	TRACE("packageDeclarationCS", "Package ", pathname);	  	
		}			
	  	( contextDeclarationCS[defaultPackage, constraints] )+	  	
	  	"endpackage"	  
	|
		( contextDeclarationCS[defaultPackage, constraints] )+	
	)
	EOF )
;
	

/*
 * pathNameCS     name :: name :: name
 */
pathNameCS returns  [Vector sequenceOfNames]  
{
	sequenceOfNames = null;
	String simpleName; 
} :
	
	/*( t:NAME 
		( "::"  sequenceOfNames = pathNameCS )?
    ) */
    (	
    	p:PATHNAME
    	{
    		StringTokenizer tokenizer = new StringTokenizer(p.getText(), "::");
    		sequenceOfNames = new Vector();
    		while (tokenizer.hasMoreTokens()) {
    			sequenceOfNames.add(tokenizer.nextToken());
    		}
    	}
    |
    	t:NAME
    	{
    		sequenceOfNames = new Vector();
    		sequenceOfNames.add(t.getText());
    	}
    |
    	n:NUMERICPATHNAME
    	{
    		StringTokenizer tokenizer = new StringTokenizer(n.getText(), "::");
    		sequenceOfNames = new Vector();
    		while (tokenizer.hasMoreTokens()) {
    			sequenceOfNames.add(tokenizer.nextToken());
    		}
    	}
    )

;


   
/*
 * contextDeclarationCS
 *   omitting:  attrOrAssocContextCS and operationContextCS 
 */
contextDeclarationCS[EPackage defpack, EList constraints] :	
 	(  
 		classifierContextDeclCS[defpack, constraints] 
 	)
;

 
/*
 * classifierContextDeclCS
 */
classifierContextDeclCS[EPackage defpack, EList constraints] 
{ 	EClassifier type = null;
 	Vector pathName = null; 
 	Constraint astNode = null;
 	String instanceVarName = null;
}  :	
 	(
 		"context" (c:NAME COLON)? pathName = pathNameCS 
 		{   Environment env = new EnvironmentImpl(defpack);
 			type = env.lookupPathName(pathName);
 			if (type == null) 
 				ERROR("classifierContextDeclCS", "Unrecognized context: " + 
 								makeString(pathName));
 			if (c != null && c.getText().length() > 0) {
 				instanceVarName = c.getText();			
 			} else {
 				instanceVarName = "self";
 			}
 			self = genVariableDeclaration("\nclassifierContextDeclCS", env,
 							instanceVarName, type, null, true, true);
 								
  			TRACE("classifierContextDeclCS", "context", pathName);  
 		}
 			
 		( astNode = invOrDefCS[env] 
 			{ astNode.setInstanceVarName(instanceVarName);
 			  constraints.add(astNode); }
 		)+
 		
 	)
;
    
/*
 * invOrDefCS
 */
invOrDefCS[Environment env] returns [Constraint astNode]
{ 	OclExpression oclexpr = null;
	astNode = null;
} :
	(
		"inv" ( n:NAME )? COLON oclexpr = oclExpressionCS[env] 		
		{  
			/* create a constraint astNode  -- must reference the type of self...
		       also, can have a name n.  type of constraint is inv...
		       constraints can be of different types ??  inv or pre/post?
		    */
			astNode = umlFactory.createConstraint();

			if (n != null) {
				astNode.setName(n.getText());
			}
			
			EClassifier type = self.getType();
			EList constrainedElement = astNode.getConstrainedElement();
			constrainedElement.add(type);
			astNode.setBody(oclexpr);
		}
	)
;
	

/*
 * oclExpressionCS -- implies
 */
oclExpressionCS[Environment env] returns [OclExpression astNode]
{ 	
	OclExpression expr = null;
	astNode = null;
	EList args = null;		 
} :
	( 
	    astNode = oclExpr1CS[env]
	    (options { warnWhenFollowAmbig = false; } :
		  	
			( "implies" expr = oclExpr1CS[env] ) 
		  	
			{ 
		 		args = new BasicEList();
		 		args.add(expr);
		 		astNode = genOperationCallExp(env, "oclExpr2CS", "implies", astNode, args);	      
		 	}
		)*
	)
;

/*
 * oclExpressionCS -- and, or
 */
oclExpr1CS[Environment env] returns [OclExpression astNode]
{
	OclExpression expr = null;
	astNode = null;
	EList args = null;
	String operName = null;
} :
	(   
		 astNode = oclExpr2CS[env]
		 ( options { warnWhenFollowAmbig = false; } : 
			( 
				("and") 	{ operName = "and"; } 
			| 
		   	  	("or") 		{ operName = "or"; } 
		   	|
		   	 	( "xor")    { operName = "xor"; }
		   	)
		   	expr = oclExpr2CS[env] 
		   	{
		   		args = new BasicEList();
		   		args.add(expr);
		       	astNode = genOperationCallExp(env, "oclExpr1CS", operName, astNode, args);
	    	
		   	}
		   				   	  
		)*
	)
;


/*
 * oclExpressionCS -- equality  =, <>
 */
oclExpr2CS[Environment env] returns [OclExpression astNode]
{
	OclExpression expr = null;
	astNode = null;
	EList args = null;
	String operName = null; 							   	    
} :
	(
		// TBD:  take care of case when an operand has an EDataType type...
		astNode = oclExpr3CS[env]
		( options { warnWhenFollowAmbig = false; } :
			( 	
				(EQUALS)  { operName = "oclEquals"; }
			|
				(NEQUALS) { operName = "oclNotEquals"; }
			|
		
				(GT) 	 { operName = "greaterThan"; }
			|
				(GE)  	 { operName = "greaterThanEqual"; }
			| 
				(LT)  	 { operName = "lessThan"; }
			|
				(LE)	 { operName = "lessThanEqual"; }
			)
			expr = oclExpr3CS[env]				
			{
				args = new BasicEList();
				args.add(expr);
				astNode = genOperationCallExp(env, "oclExpr2CS", operName, astNode, args);
	    	
			}
			
		)*
	)
;


/*
 * oclExpression -- if-then-else-endif
 */
oclExpr3CS[Environment env] returns [OclExpression astNode]
{ 	astNode = null; }  :
	
	( 	astNode = ifExpCS[env] 
	|	astNode = oclExpr4CS[env]
	)
;


/*
 * ifExpCS
 */
ifExpCS[Environment env] returns [OclExpression astNode]
{ 	astNode = null; 
	OclExpression expr1 = null;
	OclExpression expr2 = null;
	OclExpression expr3 = null;
	IfExp ifNode = null;
} :
	
	( "if" expr1 = oclExpressionCS[env]
	  "then"  expr2 = oclExpressionCS[env]
	  "else"   expr3 = oclExpressionCS[env]
	  "endif"
	)
	{ 
		TRACE("ifExpCS", " "); 
		
		if (expr1.getType() != PrimitiveTypeImpl.OCL_BOOLEAN && 
							expr1 != PrimitiveTypeImpl.OCL_VOID) {
			ERROR("ifExpCS", "expect boolean condition for if statement");
		}
	
		ifNode = expressionsFactory.createIfExp();
		ifNode.setCondition(expr1);
		ifNode.setThenExpression(expr2);
		ifNode.setElseExpression(expr3);
		ifNode.setType(AnyTypeImpl.commonSuperType(expr2.getType(), expr3.getType()));
		astNode = ifNode;
	}
;

/*
 * oclExpressionCS -- +, -
 */
oclExpr4CS[Environment env] returns [OclExpression astNode]
{
	OclExpression expr = null;
	astNode = null;
	String operName = null;
	EList args = null;
} :
	(
		astNode = oclExpr5CS[env] 
			
		( options { warnWhenFollowAmbig = false; } :
			(
				(PLUS)  { operName = "plus"; }  
			| 
		   	  	(MINUS) 
		   	  	{ 
		   	  		if (astNode.getType() instanceof CollectionType) {
		   	  			operName = "symmetricDifference";
		   	  		} else {
		   	  			operName = "minus";
		   	  		} 
		   	  	}
		   	  	 
		   	)
		   	expr = oclExpr5CS[env]
		   	{
		   		args = new BasicEList();
		   		args.add(expr);
		   		astNode = genOperationCallExp(env, "oclExpr4CS", operName, astNode, args);	    	
	    	}
		)*
	)
;


/*
 * oclExpressionCS -- *, /
 */
oclExpr5CS[Environment env] returns [OclExpression astNode]
{
	OclExpression expr = null;
	astNode = null;
	EOperation oper = null;
	EList args = null;
	String operName = null;
} :
	(
		astNode = oclExpr6CS[env] 
		( options { warnWhenFollowAmbig = false; } :
			(
				(MULTIPLY) { operName = "times"; }    	   
			| 
				(DIVIDE)   { operName = "divide"; } 
			)
			expr = oclExpr6CS[env]
			{
				args = new BasicEList();
				args.add(expr);
				astNode = genOperationCallExp(env, "oclExpr5CS", operName, astNode, args);
	    	
			}
		)*
	)
;
			
/*
 * oclExpressionCS -- unary minus, not 
 */
oclExpr6CS[Environment env] returns [OclExpression astNode]
{
	OclExpression expr = null;
	astNode = null;
	String operName = null;
} :
	(
		astNode = oclExpr7CS[env]
	|
		(	options { warnWhenFollowAmbig = false; } :
			(
				(MINUS) 
				{ 
					operName = "minus";
				}
			|
				("not")  { operName = "not"; }				
			)
			expr = oclExpr6CS[env]			
			{ 
				astNode = genOperationCallExp(env, "oclExpr6CS", operName, expr, new BasicEList());
			}
		)

	)
;


/*
 * oclExpressionCS  dot and arrow 
 */
oclExpr7CS[Environment env] returns [OclExpression astNode]
{
	astNode = null;
} :
	(
		astNode = oclExprCS[env, null]
		astNode = dotOrArrowExtension[env, astNode]
		
	)
;
	

/*
 * dotOrArrowExtension   (  .a  | ->b )*
 */
dotOrArrowExtension[Environment env, OclExpression source] returns [OclExpression astNode]	 
{
	OclExpression expr = null;
	astNode = source;
}  :

	( options { warnWhenFollowAmbig = false; } :
		(
			  ( DOT expr = modelPropertyCallExpCS[env, astNode] )
			  { 
			  	TRACE("dotOrArrowExtension", "dot"); 
			  }  
		| 
		   	  ( ARROW )	   	  	
		   	  {	
		   	  	/* The source must be a collection type.
		   	  	 */
		   	  	if (!(astNode.getType() instanceof CollectionType)) {
		   	  		CollectionLiteralExp astNode1 = expressionsFactory.createCollectionLiteralExp();
					astNode1.setKind(CollectionKind.SET_LITERAL);
					EList collectionParts = astNode1.getParts();
					CollectionItem collItem = expressionsFactory.createCollectionItem();
 					collItem.setType(astNode.getType());
 					collItem.setItem(astNode);				
					collectionParts.add(collItem);
		   	  		SetType type = typesFactory.createSetType(astNode.getType());
		   	  		type.setName("Set(" + type.getName() + ")" );
		   	  		astNode1.setType(type);
		   	  		astNode = astNode1;
		   	  	}	
		   	  		
		   	  } 
		   	  ( expr = setExpCS[env, astNode] ) 
		   	  { 
		   	  	TRACE("dotOrArrowExtension", "arrow"); 
		   	  }
		   	  	
		 )
		 { 
		      astNode = expr;		              	 
		 } 
	)*
;

							
/*
 * OclExprCS
 */
oclExprCS[Environment env, OclExpression source] returns [OclExpression astNode]   

	{  astNode = null; } :
	
	( 
		astNode = letExpCS[env] |		
		astNode = operationCallExpCS[env, source, false] |
		astNode = literalExpCS[env] |
		astNode = simpleVariableCS[env, source] (PRE)? |
      	LPAREN astNode = oclExpressionCS[env] RPAREN  
	)
;

/*
 * modelPropertyCallExpCS  -- after "." can have an operation call or a property reference
 */
modelPropertyCallExpCS[Environment env, OclExpression source] returns [OclExpression astNode]
	{ astNode = null; } :
	( 
		( astNode = operationCallExpCS[env, source, true]  )
	|
		( astNode = simpleVariableCS[env, source] )
	)
;

/*
 * oclSetExpCS --  expressions after ->
 * type is the type of the source
 */
setExpCS[Environment env, OclExpression source] returns [OclExpression astNode]
	{ 	astNode = null; } :
	(  
		( astNode = iteratorExpCS[env, source] )
	|
		( astNode = iterateExpCS[env, source] )
	|
		( astNode = operationCallExpCS[env, source, false] )
	
	)
;



/*
 * LetExpCS
 */
letExpCS[ Environment env] returns [LetExp astNode]  
	{ 
	  VariableDeclaration varDeclaration;
	  OclExpression letSubExp;
	  String varName = null;	  
	  astNode = null;
    }  :

    ( "let" varDeclaration = variableDeclarationCS[env, true]
    	{
    		if (varDeclaration.getType() == null) {
				ERROR("letExpCS", "Missing type for variable " + varName);
	   		}
	   		if (varDeclaration.getInitExpression() == null) {
				ERROR("letExpCS", "Missing init expression for variable " + varName);
	   		} 
    	}
	   	letSubExp = letExpSubCS[env] 
	)

	{
	   
	   varName = varDeclaration.getVarName();
	   astNode = expressionsFactory.createLetExp(); 
	   astNode.setVariable(varDeclaration);
	   astNode.setIn(letSubExp);
	   astNode.setType(letSubExp.getType());
	   env.deleteElement(varName);
	}
;


/*
 * VariableDeclarationCS
 */
variableDeclarationCS[Environment env, boolean addToEnvironment] returns [VariableDeclaration astNode] 
   	{
	  	String varName;
	  	EClassifier type = null;
	  	OclExpression expr = null;
	  	
	  	astNode = null;
   	}  :
   	
   	
   	(   t:NAME 
   		( COLON type = typeCS[env]
   			{ if (type == null) 
   				ERROR("variableDeclarationCS", "Unrecognizable type for variable: " + t.getText());
   			}
   		) ?	

	  	( options { warnWhenFollowAmbig = false; } :
	  	EQUALS  expr = oclExpressionCS[env]) ?  
   	) 
	{
		varName = t.getText();
		astNode = genVariableDeclaration("variableDeclarationCS", env, varName, type, expr, 
						true, addToEnvironment);
	}
;

/*
 * TypeCS
 */
typeCS[Environment env] returns [EClassifier astNode] 
{
     astNode = null;
	Vector pathName = null;
} :  
	( 	
		astNode = primitiveTypeCS
	|
		pathName = pathNameCS 
		{
			astNode = env.lookupPathName(pathName);
		}
	|
		astNode = collectionTypeCS[env]
	|
		astNode = tupleTypeCS[env]
	| 
		LPAREN astNode = typeCS[env] RPAREN
	)
;

/**
 * primitiveTypeCS
 */
primitiveTypeCS returns [EClassifier astNode]
{ astNode = null;
} :
	(	"Integer"	{
						astNode = PrimitiveTypeImpl.OCL_INTEGER; }
	|	"String"	{ 
					 astNode = PrimitiveTypeImpl.OCL_STRING;}
	|	"Real"		{
					 astNode = PrimitiveTypeImpl.OCL_REAL; }
	|	"Boolean"	 {
						astNode = PrimitiveTypeImpl.OCL_BOOLEAN; }
	|   "OclAny"
					 {  astNode = AnyTypeImpl.OCL_ANY_TYPE; }
	|	"OclVoid"
					{  astNode = AnyTypeImpl.OCL_VOID; }
	)
;

/*
 * collectionTypeCS
 */
collectionTypeCS[Environment env] returns [CollectionType astNode]
{	astNode = null;
	EClassifier type = null;
} :
	( astNode = collectionTypeIdentifierCS LPAREN
		type = typeCS[env] RPAREN
	)
	{
		astNode.setElementType(type);
		astNode.setName(astNode.getName() + "(" + type.getName() + ")" );
	}
;

/*
 * collectionTypeIdentifierCS 
 */
collectionTypeIdentifierCS returns [CollectionType astNode]
{	astNode = null;
} :
	(
		"Set"
		{
			astNode = typesFactory.createSetType();
			astNode.setName("Set");
			TRACE("collectionTypeIdentifierCS", "SET");
		}
		|	"Bag"
		{
			astNode = typesFactory.createBagType();
			astNode.setName("Bag");  
			TRACE("collectionTypeIdentifierCS", "BAG"); 
		}
		|	"Sequence"
		{  	
			astNode = typesFactory.createSequenceType();
			astNode.setName("Sequence");
			TRACE("collectionTypeIdentifierCS", "SEQUENCE"); 
		}
		|	"Collection"
		{
			astNode = typesFactory.createCollectionType();
			astNode.setName("Collection"); 
			TRACE("collectionTypeIdentifierCS", "COLLECTION"); 
		}
		| 	"OrderedSet"
		{
			astNode = typesFactory.createOrderedSetType();
			astNode.setName("OrderedSet");
			TRACE("collectionTypeIdentifierCS", "ORDERED_SET"); 
		}
	)
	{ astNode.setElementType(AnyTypeImpl.OCL_VOID); }
;



/*
 * tupleTypeCS
 */
tupleTypeCS[Environment env] returns [EClassifier astNode]
{
	astNode = null;
	VariableDeclaration vdcl = null;
	HashSet names = null;
	EList vdcls = null;
	String nodeName = null;
} :
	( "Tuple" LPAREN 
		{ 
			TRACE("typeTypeCS", "Tuple Type");
			names = new HashSet();
			vdcls = new BasicEList();		
		}						
		( 	
			vdcl = variableDeclarationCS[env, false]
			{
				String name = vdcl.getVarName();
				TRACE("tupleTypeCS", " name = " + name);
				names.add(name);
				if (vdcl.getInitExpression() != null) {
					ERROR("tupleTypeCS", "Initialize expressions not allowed in types: " + name);
				}
				if (vdcl.getType() == null) {
					ERROR("tupleTypeCS", "Missing type declaration for " + name);
				}  else {
					vdcls.add(vdcl);
					nodeName = "Tuple(" + vdcl.getVarName() + ":" + vdcl.getType().getName();
					
				}										
			}
			( 	options { warnWhenFollowAmbig = false; } :
				COMMA  vdcl = variableDeclarationCS[env, false]
				{
					name = vdcl.getVarName();
					TRACE("tupleTypeCS", " name = " + name);
					if (names.contains(name)) {
						ERROR("tupleTypeCS", "Duplicate name in tuple " + name);
						vdcl.setVarName(null);
					} else {
						names.add(name);
					}
					
					if (vdcl.getInitExpression() != null) {
						ERROR("tupleTypeCS", "Initialize expressions not allowed in types: " + name);
					}
					if (vdcl.getType() == null) {
						ERROR("tupleTypeCS", "Missing type declaration for " + name);
					}
					vdcls.add(vdcl);
					nodeName += ", " + vdcl.getVarName() + ":" + vdcl.getType().getName();					
				}
			)*
		) ?
		RPAREN
		{
			astNode = typesFactory.createTupleType(vdcls);
			astNode.setName(nodeName + ")");
		}
	)
;
 
		

/*
 * LetExpSubCS
 */
letExpSubCS[ Environment env] returns [OclExpression astNode] 
{
	VariableDeclaration varDeclaration;
	OclExpression letSubExp;
	String varName = null;
	LetExp letExp;		
	astNode = null;
}  :

	( 	COMMA varDeclaration = variableDeclarationCS[env, true]
		{
			if (varDeclaration.getType() == null) {
				ERROR("letExpSubCS", "Type not provided for " + varName);
			}
			if (varDeclaration.getInitExpression() == null) {
				ERROR("letExpSubCS", "Init expr not provided for "+ varName);
			}
		} 
		letSubExp = letExpSubCS[env]  
	) 
	 
	{  
		varName = varDeclaration.getVarName();
		env.deleteElement(varName);

		letExp = expressionsFactory.createLetExp(); 
		letExp.setVariable(varDeclaration);
		letExp.setIn(letSubExp);
		letExp.setType(letSubExp.getType());		
		astNode = letExp;
	}
	|
	( "in" astNode = oclExpressionCS[env] )
;



/*
 * simpleVariableExpCS
 */
simpleVariableCS[Environment env, OclExpression source] returns  [OclExpression astNode]   
	{ 
		String simpleName;
		
		astNode = null;
		EAttribute eattr = null;
		EReference eref = null;
		EClassifier classifier = null;
		AttributeCallExp attr = null;
		AssociationEndCallExp ref = null;
		VariableDeclaration vdcl = null;
	} :
	
	( t:NAME )
	{
		
		/* A name can be a variable defined by a Variable declaration, the special
		  variable "self", an attribute or a reference to another object.
		  If the source is not null, then the last token was a "." or "->"
		  The source is used to establish the navigation.
		  If no type is provided, then either the name is a the use of a variable,
		  or there is an implicit variable declaration (self or an iterator)
		  that is the source.		  		   
		 */
		simpleName = t.getText();
		
		/*
		 * The source may be a collection type (for example, in self.children.name, children
		 * may be a set.)_  In this case, we have to get the element type of children, so
		 * that the attribute name can be found.
		 * The source type can also be a tuple type. In this case, we need to get the 
		 * EClass of the tuple.
		 * 
		 */ 
		EClassifier sourceElementType = null;
		if (source != null) {
			sourceElementType = source.getType();
			if (sourceElementType instanceof CollectionType) {
				sourceElementType = ((CollectionType) sourceElementType).getElementType();
			} 
		}
		
		Vector names = new Vector();
		names.addElement(simpleName);
		if ((classifier = env.lookupPathName(names)) != null) {
			/* Variable is a classifier. Classifiers are used in
			 * allInstances, isKindOf, isTypeOf, asTypeOf operators
			 * It is not clear how to represent types as OclExpressions,
			 * so we generate an UnspecifiedValueExpression with correct type.
			 */
			UnspecifiedValueExp vexp = expressionsFactory.createUnspecifiedValueExp();	
			vexp.setType( classifier);
			astNode = vexp;			 		
		} else if (source == null && (vdcl = env.lookup(simpleName)) != null)  { 
			// Either a use of a declared variable or self

			TRACE("variableExpCS", "Variable Expression: " + simpleName);
			
			/*if (source != null) {
				ERROR("simpleVariableExp", "Can't have a variable in the middle of a navigation path:"
								+ simpleName);
			}*/
			VariableExp vexp = expressionsFactory.createVariableExp();	
			vexp.setReferredVariable(vdcl);
			vexp.setName(vdcl.getVarName());
			vexp.setType( vdcl.getType());
			astNode = vexp;
		} else if ((eattr = env.lookupAttribute( sourceElementType, simpleName)) != null) {
			
			TRACE("variableExpCS", "Attribute: " + simpleName);
			attr = expressionsFactory.createAttributeCallExp();
			attr.setReferredAttribute(eattr);
			attr.setType(env.getOclTypeOrSequenceType( eattr.getEAttributeType(), 
									eattr.isMany()));
			if (source != null) {
				attr.setSource(source);
			} else {
				VariableExp src = expressionsFactory.createVariableExp();
				VariableDeclaration implicitSource = env.lookupImplicitSourceForAttribute(simpleName);
				src.setType(implicitSource.getType());
				src.setReferredVariable(implicitSource);
				src.setName(implicitSource.getVarName());
				attr.setSource(src);
			}
			astNode = attr;
						
		} else if ((eref = env.lookupReference( sourceElementType, simpleName)) != null) {
			TRACE("variableExpCS", "Reference: " + simpleName);
			ref = expressionsFactory.createAssociationEndCallExp();
			ref.setReferredAssociationEnd(eref);
			ref.setType(env.getOclTypeOrSequenceType(eref.getEReferenceType(), eref.isMany()));
			
			if (source != null) {
				ref.setSource(source);
			} else {
				VariableExp src = expressionsFactory.createVariableExp();
				VariableDeclaration implicitSource = env.lookupImplicitSourceForAssociationEnd(simpleName);
				src.setType(implicitSource.getType());
				src.setReferredVariable(implicitSource);
				src.setName(implicitSource.getVarName());
				ref.setSource(src);
			}
			astNode = ref;
									
		} else {
			if (source != null && (vdcl = env.lookup(simpleName)) != null) 
				ERROR("variableExpCS", "Variable cannot appear in the middle of a navigation expression: " + simpleName);
		 	else 
				ERROR("variableExpCS", "unrecognized variable: " + simpleName);		
		}
		
		/*
		 * If the source type is a collection, then need there is an implicit COLLECT operator.
		 * Not that this rule is not called after "->".
		 */
		if (source != null && source.getType() instanceof CollectionType) {
			IteratorExp iterexp = expressionsFactory.createIteratorExp();
			
			VariableDeclaration itervar = genVariableDeclaration("VariableExpCS", env,
							null, sourceElementType, null, false, true );

			EList iters = iterexp.getIterators();
			iters.add(itervar);
			iterexp.setBody(astNode);
			iterexp.setName("collect");
			VariableExp vexp = expressionsFactory.createVariableExp();
			vexp.setType(itervar.getType());
			vexp.setReferredVariable(itervar);
			vexp.setName(itervar.getVarName());
			/* adjust the source variable for the body expression  to be the
			   newly generated implicit iterator variable */
			if (astNode instanceof AttributeCallExp) {
				attr.setSource(vexp);
			} else if (astNode instanceof AssociationEndCallExp) {
				ref.setSource(vexp);
			}
			iterexp.setSource(source);
			if (source.getType() instanceof SequenceType ||
					source.getType() instanceof OrderedSetType) {
				iterexp.setType(typesFactory.createSequenceType(astNode.getType()));
			} else {
				iterexp.setType(typesFactory.createBagType(astNode.getType()));
			}
			astNode = iterexp;			
			env.deleteElement(itervar.getVarName());			
	   	}
	}		
;


/*
 * LiteralExpCS 
 */
literalExpCS[Environment env] returns [OclExpression astNode] 
{	astNode = null;
} : 
	( 
		astNode = primitiveLiteralExpCS[env] 
	|	astNode = collectionLiteralExpCS[env]
	| 	astNode = tupleLiteralExpCS[env]
	| 	astNode = enumerationOrClassLiteralExpCS[env]
	)
;


/*
 * enumerationOrClassLiteralExpCS
 */
enumerationOrClassLiteralExpCS[Environment env] returns [OclExpression astNode]
{ 	astNode = null;
	Vector sequenceOfNames = null;
	String lastToken = null;
	EClassifier prim = null;
 
} :
	( t:PATHNAME )
	{
    	StringTokenizer tokenizer = new StringTokenizer(t.getText(), "::");
    	sequenceOfNames = new Vector();
    	while (tokenizer.hasMoreTokens()) {
    		lastToken = tokenizer.nextToken();
    		if (tokenizer.hasMoreTokens()) {
    			sequenceOfNames.add(lastToken);
    		}
    	}	
 		EEnumLiteral literal = null;		
		EClassifier enumType = env.lookupPathName(sequenceOfNames);
		if (enumType == null || ! (enumType instanceof EEnum)) {
			astNode = expressionsFactory.createUnspecifiedValueExp();
			
			// Check to see whether the pathname corresponds to a type
			sequenceOfNames.add(lastToken);
			EClassifier type = env.lookupPathName(sequenceOfNames);
			if (type == null) {		
				ERROR("enumerationOrClassLiteralExpCS", "Unrecognizable package or enum ", sequenceOfNames);
			} else {
				astNode.setType(type);
			}		
		} else { // is an enum
			astNode = expressionsFactory.createEnumLiteralExp();
			literal = ((EEnum) enumType).getEEnumLiteral(lastToken);
			if (literal == null) {
				ERROR("enumerationOrClassLiteralExpCS", "Unknown enumeration literal " +
									lastToken);
			}
			EnumLiteralExp litExp = (EnumLiteralExp) astNode;
			litExp.setReferredEnumLiteral(literal);
			astNode = litExp;
			if (enumType == null || literal == null) {
				astNode.setType(PrimitiveTypeImpl.OCL_VOID);
			} else {
				astNode.setType(enumType);
			}			
		}
		TRACE("enumerationOrClassLiteralExpCS", t.getText());	
	}	
;
	

/*
 * PrimitiveLiteralExpCS
 */
primitiveLiteralExpCS[Environment env] returns [LiteralExp astNode]
{ astNode = null;
} :
	
	( 	
		astNode = integerLiteralExpCS[env]
	|
		astNode = realLiteralExpCS[env]
	|
		astNode = stringLiteralExpCS[env]
	|
		astNode = booleanLiteralExpCS[env]
	)
;


/*
 * IntegerLiteralExpCS
 */
integerLiteralExpCS[Environment env] returns [IntegerLiteralExp astNode]
	{ astNode = null; } :
	
	( 	t:NUMBER_LITERAL )
	{ 	
		astNode = expressionsFactory.createIntegerLiteralExp();
		astNode.setIntegerSymbol(new Integer(t.getText()));
		astNode.setType(PrimitiveTypeImpl.OCL_INTEGER);
		astNode.setName("Integer");
		TRACE("integerLiteralExpCS", "Integer: " + t.getText());
	}
;

/*
 * RealLiteralExpCS
 */
realLiteralExpCS[Environment env] returns [RealLiteralExp astNode]
{ astNode = null; 
} :
	( t:REAL_LITERAL )
	{
		astNode = expressionsFactory.createRealLiteralExp();
		astNode.setRealSymbol(new Double(t.getText()));
		astNode.setType(PrimitiveTypeImpl.OCL_REAL);
		astNode.setName("Real");
		TRACE("realLiteralExpCS", "Real: " + t.getText());
	}
;


/*
 * BooleanLiteralExpCS
 */
booleanLiteralExpCS[Environment env] returns [BooleanLiteralExp astNode]
{ 	astNode = null; 
	boolean result;
} :
	(
		"true"    
		{ 
			astNode = expressionsFactory.createBooleanLiteralExp();
			astNode.setBooleanSymbol(Boolean.TRUE);
		}
	|
		"false"   
		{ 
			astNode = expressionsFactory.createBooleanLiteralExp(); 
			astNode.setBooleanSymbol(Boolean.FALSE);
		}
	)
	{
		astNode.setType(PrimitiveTypeImpl.OCL_BOOLEAN);
		astNode.setName("Boolean");
		TRACE("boleanLiteralExpCS", "Boolean");
	}
;


/*
 * StringLiteralExpCS
 */
stringLiteralExpCS[Environment env] returns [StringLiteralExp astNode]
{ astNode = null; 
} :
	( t:STRING_LITERAL )
	{
		astNode = expressionsFactory.createStringLiteralExp();
		String stringLiteral = t.getText();
		if (stringLiteral.length() <= 2) {
			astNode.setStringSymbol("");
		} else {
			astNode.setStringSymbol(stringLiteral.substring(1, stringLiteral.length()-1));
		}
		astNode.setType(PrimitiveTypeImpl.OCL_STRING);
		astNode.setName("String");
		TRACE("stringLiteralExpCS", "String: " + t.getText());
	}
;


/*
 * CollectionLiteralExpCS
 */
collectionLiteralExpCS[Environment env] returns [CollectionLiteralExp astNode]
{
	astNode = null;
	CollectionKind kind = null;
	CollectionLiteralPart collectionLiteralPartExp = null;
	EList collectionParts = null;
	EClassifier type = null;
	CollectionType resultType = null;
} :
	(
		resultType = collectionTypeIdentifierCS		 
		{	if (resultType instanceof SetType)
				kind = CollectionKind.SET_LITERAL; 
			else if (resultType instanceof BagType)
				kind = CollectionKind.BAG_LITERAL;
			else if (resultType instanceof  SequenceType)
				kind = CollectionKind.SEQUENCE_LITERAL;
			else if (resultType instanceof OrderedSetType)
				kind = CollectionKind.ORDERED_SET_LITERAL;
									
			astNode = expressionsFactory.createCollectionLiteralExp();
			astNode.setKind(kind);
			collectionParts = astNode.getParts();
		}					   
		LBRACKET 
		(
			collectionLiteralPartExp = collectionLiteralPartCS[env]
			{ 
				collectionParts.add(collectionLiteralPartExp);				
				type = collectionLiteralPartExp.getType();				
			}			
			( COMMA collectionLiteralPartExp = collectionLiteralPartCS[env]
				{ 
					EClassifier type1 = collectionLiteralPartExp.getType();					
					type = AnyTypeImpl.commonSuperType(type, type1);
					collectionParts.add(collectionLiteralPartExp);
				}
			)*
		)? 
		RBRACKET
		{  
			
			if (collectionParts.isEmpty()) {
				resultType.setElementType(PrimitiveTypeImpl.OCL_VOID);
				resultType.setName(resultType.getName() + "()");
			} else {
				resultType.setElementType(type);
				resultType.setName(resultType.getName() + "(" + type.getName() + ")" );
			}
			astNode.setType(resultType);
			 
		}
					
	)
;


/*
 * collectionLiteralPartCS
 */
collectionLiteralPartCS[Environment env] returns [CollectionLiteralPart astNode]
{
 	astNode = null;
 	OclExpression expr1 = null;
 	OclExpression expr2 = null;
 	CollectionRange collRange = null;
 	CollectionItem collItem = null;
} :
 	(
 		(
 			expr1 = oclExpressionCS[env]
 			( DOTDOT expr2 = oclExpressionCS[env] 
 				{
 					collRange = expressionsFactory.createCollectionRange();
 					collRange.setFirst(expr1);
 					collRange.setLast(expr2);
 					collRange.setType(expr1.getType());
 					// TODO:  What types are allowed in ranges.  Only Integers?
 					if (expr1.getType() != expr2.getType()) {
 						ERROR("collectionLiteralPartCS", "First and last type mismatch");
 					}
 					astNode = collRange;
 					TRACE("collectionLiteralPartCS", "collection range");
 				}
 			) ?
 		) 
 	)
 		
 	{ 
 		if (expr2 == null) {
 			collItem = expressionsFactory.createCollectionItem();
 			collItem.setType(expr1.getType());
 			collItem.setItem(expr1);
 			astNode = collItem;
 			TRACE("collectionLiteralPartCS", "collection item");
 		}
 	}
;


/*
 * tupleLiteralExpCS
 */
tupleLiteralExpCS[Environment env] returns [TupleLiteralExp astNode]
{
	astNode = null;
	VariableDeclaration vdcl = null;
	EList tupleParts = null;
	EList vdcls = null;
	HashSet names = null;
	String typeName = null;
} :
	(
		"Tuple" LBRACKET
		{ 
			astNode = expressionsFactory.createTupleLiteralExp();
			tupleParts = astNode.getTuplePart();
			vdcls = new BasicEList();
			TRACE("tupleLiteralExpCS", "Tuple");
			typeName = "Tuple (";
		}
		vdcl = variableDeclarationCS[env, false]
		{ 
			tupleParts.add(vdcl); 
			vdcls.add(vdcl);
			
			String name = vdcl.getVarName();
			names = new HashSet();		
			names.add(name);
			TRACE("tupleLiteralExpCS", " name = " + name);
			if (vdcl.getType() == null) {
				ERROR("tupleLiteralExpCS", "Missing type declaration for " + name);
			}
			if (vdcl.getInitExpression() == null) {
				ERROR("tupleLiteralExpCS", "Missing initializer for " + name);
			}
			typeName += vdcl.getVarName() + ":" + vdcl.getType().getName();
		}
		( COMMA vdcl = variableDeclarationCS[env, false] 
			{ 
				tupleParts.add(vdcl);
				vdcls.add(vdcl);
				name = vdcl.getVarName();
				TRACE("tupleLiteralExpCS", " name = " + name);
				if (vdcl.getType() == null) {
					ERROR("tupleLiteralExpCS", "Missing type declaration for " + name);
				}
				if (vdcl.getInitExpression() == null) {
					ERROR("tupleLiteralExpCS", "Missing initializer for " + name);
				}
				if (names.contains(name)) {
					ERROR("tupleLiteralExpCS", "Duplicate name in tuple " + name);
					vdcl.setVarName(null);
				} else {
					names.add(name);
				}
				typeName += ", " + vdcl.getVarName() + ":" + vdcl.getType().getName();
			}
		) *
		RBRACKET
		{ 
			TupleType tt = typesFactory.createTupleType(vdcls);
			tt.setName(typeName + "}");
			astNode.setType(tt);
		}
	)
;
 		
 
/*
 * iteratorExpCS
 */
iteratorExpCS[Environment env, OclExpression source] returns [IteratorExp astNode]
{ 
	astNode = null; 
	VariableDeclaration vdcl = null;
	VariableDeclaration vdcl1 = null;
	OclExpression expr = null;
	String name = null;
	EList iterators = null;
} :
	(    
		(   /* Collection iterators */
		   	"forAll"   	
		   	{ name = "forAll"; } 
		|  	"exists"   	
			{ name = "exists";}
		|  	"isUnique" 
			{ name = "isUnique"; } 
		|  	"any"      	
			{ name = "any";  }
		|  	"one"      	
			{ name = "one"; }
		|  	"collect"  	
			{ name = "collect"; } 
		|  	"select"   	
			{ name = "select"; }
		|	"reject"	
			{ name = "reject";   }
		|	"collectNested"	
			{name = "collectNested"; }
		|	"sortedBy"	
			{ name = "sortedBy"; }	
			 
		) 
		LPAREN astNode = iterContents[env, name, source] RPAREN
	)
;

/*
 *iterContents
 */
iterContents[Environment env, String name, OclExpression source] returns [IteratorExp astNode]
{ 
	astNode = null; 
	VariableDeclaration vdcl = null;
	VariableDeclaration vdcl1 = null;
	OclExpression expr = null;
	EList iterators = null;
} :
	(
		{ LT(2).getText().equals(":") || LT(2).getText().equals(",")
   	 	 || LT(2).getText().equals(";") || LT(2).getText().equals("|") }?
   	 	  				 
		( options { warnWhenFollowAmbig = false; } :
			(vdcl = variableDeclarationCS[env, true] 
			{	
				astNode = expressionsFactory.createIteratorExp();
				astNode.setName(name);
				iterators = astNode.getIterators();	
				if (vdcl.getType() == null) {
					EClassifier sourceType = source.getType();
					if (sourceType instanceof CollectionType) {
						vdcl.setType(((CollectionType) sourceType).getElementType());
					}
				}
				iterators.add(vdcl);
			}
	  		( 
	  			COMMA vdcl1 = variableDeclarationCS[env, true] 
	  			{
	  				if (vdcl1.getType() == null) {
						EClassifier sourceType = source.getType();
						if (sourceType instanceof CollectionType) {
							vdcl1.setType(((CollectionType) sourceType).getElementType());
						}
	  				}
	  				iterators.add(vdcl1);
	  			} 
	  	 	) ?
	  		BAR
	  		) 	  	
	  		expr = oclExpressionCS[env] 
		) 
	|		
		{ 
			astNode = expressionsFactory.createIteratorExp();
			astNode.setName(name);
			iterators = astNode.getIterators();	
			// Synthesize the iterator expression.
			vdcl = genVariableDeclaration("iteratorExpCS", env, null, 
				((CollectionType) source.getType()).getElementType(), null, false, true);
			iterators.add(vdcl);
		}
		( expr = oclExpressionCS[env] )
	)	  	
		
	
	{ 
		TRACE("oclIteratorExpCS: ", name);
		
		if (name.equals("forAll") || name.equals("exists") || name.equals("one") || name.equals("isUnique")) {
			astNode.setType(PrimitiveTypeImpl.OCL_BOOLEAN);
		} else if (name.equals("select") || name.equals("reject") ) {
			astNode.setType(source.getType());
		} else if (name.equals("collect")) {
			// The result type for collect must be flattened
			EClassifier elementType = expr.getType();
			while (elementType instanceof CollectionType) {
				elementType = ((CollectionType) elementType).getElementType();
			}
			if (source.getType() instanceof SequenceType || 
							source.getType() instanceof OrderedSetType) {
				astNode.setType(typesFactory.createSequenceType(elementType));
			} else {
				astNode.setType(typesFactory.createBagType(elementType));
			}
		} else if (name.equals("collectNested")) {
			if (source.getType() instanceof SequenceType || 
							source.getType() instanceof OrderedSetType) {
				astNode.setType(typesFactory.createSequenceType(expr.getType()));
			} else {
				astNode.setType(typesFactory.createBagType(expr.getType()));
			}
		} else if (name.equals("any")) {
			astNode.setType(((CollectionType) source.getType()).getElementType());
		} else if (name.equals("sortedBy")) {
			if ((source.getType() instanceof SequenceType) ||
					source.getType() instanceof BagType) {
				SequenceType osType = typesFactory.createSequenceType();
				osType.setElementType(((CollectionType) source.getType()).getElementType());
				astNode.setType(osType);
			} else { // set, ordered set
				OrderedSetType osType = typesFactory.createOrderedSetType();
				osType.setElementType(((CollectionType) source.getType()).getElementType());
				astNode.setType(osType);
			}
		} 
				
		astNode.setBody(expr);
		astNode.setSource(source);
		
		env.deleteElement(vdcl.getVarName());
		if (vdcl1 != null) {
			env.deleteElement(vdcl1.getVarName());
		}		
	}
;



/*
  iterateExpCS
 */
iterateExpCS[Environment env, OclExpression source] returns [IterateExp astNode]
{ 
	astNode = null; 
	VariableDeclaration vdcl = null;
	VariableDeclaration vdcl1 = null;
	OclExpression expr = null;
} :	
	(
		( 
			"iterate" LPAREN 
			vdcl = variableDeclarationCS[env, true] 
			( SEMICOLON vdcl1 = variableDeclarationCS[env, true] )? BAR
		)
		{
			/* If there is only one variable declaration, then it is the result, vdcl1. 
		   		The missing variable declaration is treated as the implicit iterator vdcl.
		 	*/
			if (vdcl1 == null) {
				vdcl1 = vdcl;
				vdcl = null;
			}	
		
			if (vdcl == null) { // synthesize a new variable declaration.		
				vdcl = genVariableDeclaration("iterateExpCS", env, null, 
						((CollectionType) source.getType()).getElementType(), null, false, true);	
			} 
			if ( vdcl.getType() == null) {
				vdcl.setType(((CollectionType) source.getType()).getElementType());
			}
		} 
		( expr = oclExpressionCS[env]   RPAREN )
	)
	{
		TRACE("iterateExpCS", "iterate");
		astNode = expressionsFactory.createIterateExp();
		astNode.setName("iterate");			
		EList iterator = astNode.getIterators();
		iterator.add(vdcl);
		astNode.setSource(source);
		astNode.setResult(vdcl1);
		astNode.setBody(expr);
		
		
		if (vdcl1.getType() == null) {
			ERROR("iterateExpCS", "Declaration for " + vdcl1.getVarName()+ " must have a type");
		}
		
		astNode.setType(vdcl1.getType());
		
		if (vdcl1.getInitExpression() == null) {
			ERROR("iterateExpCS", "Declaration for " + vdcl1.getVarName()+ 
										" must have an initial expression");
		}
		if (vdcl.getInitExpression() != null) {
			ERROR("iterateExpCS", "Declaration for " + vdcl.getVarName() +
										" must not have an initial expression");
		} 
		
		env.deleteElement(vdcl.getVarName());
		env.deleteElement(vdcl1.getVarName());
		
	}
;



/*
 * operationCallExpCS
 */
operationCallExpCS[Environment env,  OclExpression source, boolean dotOperator] returns [OclExpression astNode]
{ 
	astNode = null;
	EList args = null;
	String operationName = null; 
	IntegerLiteralExp intSource = null;
	OclExpression referenceArg = null;
	EClassifier typeArg = null;
} :
	(
		(
			( "oclAsType"  
				{ operationName = "oclAsType"; }
			)
		|
			( "oclIsKindOf"
				{ operationName = "oclIsKindOf"; }
			)
		|
			( "oclIsTypeOf" 
				{ operationName = "oclIsTypeOf"; } 			
			)
		)
		( args = operationArgs[env, operationName] )
			
	|
		(   (  "closure" 	{operationName = "closure"; } 
		    |   
			   "closureAll" { operationName = "closureAll"; }
			)
			
			 LPAREN 
			 referenceArg = modelPropertyCallExpCS[env, source]
		     referenceArg = dotOrArrowExtension[env, referenceArg]
		     { 
		     	args = new BasicEList();
		     	args.add(referenceArg); 
		     }		 		  
			 RPAREN
		)
		
	| 
		(
			// TBD:  What to do with a real pathname (length > 1) 
			t:NAME LPAREN (args = argumentsCS[env] ) ? RPAREN
		)
		{  operationName = t.getText();  }			
	|
		(	//   45.someoperation(...)
			p:OPERATION_EXPR
		)
		{
			StringTokenizer tokenizer = new StringTokenizer(p.getText(), ".");
			intSource = expressionsFactory.createIntegerLiteralExp();
			intSource.setType(PrimitiveTypeImpl.OCL_INTEGER);
			intSource.setIntegerSymbol(new Integer(tokenizer.nextToken()));
			operationName = tokenizer.nextToken();
		}
		(
			args = operationArgs[env, operationName]			
		)		
	) 
		
	{ 	
		/*
		 * The type of the operation is specified by a path expression 
		 * or self, or by an explicit source expression.  The source expression
		 * may be a collection type (-> operation) or a regular navigation expression
		 * (. operation)
		 */
		OclExpression src = source;
		if (intSource != null) {
			src = intSource;  // source is an integer:  number.name(...)
		} else if (src == null) {  // create an implicit source
			VariableExp vexp = expressionsFactory.createVariableExp();
			VariableDeclaration implicitSource = 
					env.lookupImplicitSourceForOperation(operationName, args);
			vexp.setType(implicitSource.getType());
			vexp.setReferredVariable(implicitSource);
			vexp.setName(implicitSource.getVarName());
			src = vexp;
		}
		
		astNode = genOperationCallExp(env, "operationCallExpCS", operationName, src, args);
		 
		/*
		 * If the source type is a collection and operator is ".", 
		 * then there is an implicit COLLECT operator.
		 */
		if (dotOperator && source.getType() instanceof CollectionType) {
			IteratorExp iterexp = expressionsFactory.createIteratorExp();
			
			VariableDeclaration itervar = genVariableDeclaration("operationCallExpCS", env,
				null, ((CollectionType) source.getType()).getElementType(), null, false, true );

			EList iters = iterexp.getIterators();
			iters.add(itervar);
			iterexp.setBody(astNode);
			iterexp.setName("collect");
			VariableExp vexp = expressionsFactory.createVariableExp();
			vexp.setType(itervar.getType());
			vexp.setReferredVariable(itervar);
			vexp.setName(itervar.getVarName());
			/* adjust the source variable for the body expression  to be the
			   newly generated implicit iterator variable */
			
			OperationCallExp opcallexp = (OperationCallExp) astNode;
			opcallexp.setSource(vexp);
			
			iterexp.setSource(source);
			if (source.getType() instanceof SequenceType || 
									source.getType() instanceof OrderedSetType) {
				iterexp.setType(typesFactory.createSequenceType(astNode.getType()));
			} else {
				iterexp.setType(typesFactory.createBagType(astNode.getType()));
			}
			astNode = iterexp;			
			env.deleteElement(itervar.getVarName());			
	   	}
	}
;

/*
 * operationArgs can be either an OclExpression list or an EClassifier.
 */
operationArgs[Environment env, String operName] returns [BasicEList arguments]
{ 	arguments = null;
	EClassifier typeArg;
}  :
	(
			
	{ operName.equals("oclAsType") || operName.equals("oclIsTypeOf") ||
				operName.equals("oclIsKindOf") }?	
			( LPAREN typeArg = typeCS[env] RPAREN )
			{
				if (typeArg == null) {
					ERROR("operationArgs", "Unrecognizable type");
				}
				arguments = new BasicEList();
				OclExpression arg = expressionsFactory.createUnspecifiedValueExp();
				arg.setType(typeArg);
				arguments.add(arg);
			}
		|
			( LPAREN (arguments = argumentsCS[env] ) ? RPAREN )
	)
;
			


/*
 * argumentsCS
 */
argumentsCS[Environment env] returns  [BasicEList arguments]  
{	arguments = null;
  	OclExpression expr; 
} :

	(  expr = oclExpressionCS[env]
		( COMMA   arguments= argumentsCS[env] )?
    ) 

	{      
        if (arguments == null) arguments = new BasicEList();
	    arguments.add(0, expr);	    
	}
;   


class OclLexer extends Lexer;

options { k=3; }

// NEED TO HANDLE  COMMENTS

// whitespace
//WS: 
//    ( ' '
//    | '\t'
//    | ( '\r' | '\n' ) 
//    )
//    { $setType(Token.SKIP); }
//;

// Whitespace -- ignored 
WS : ( ' ' | '\t' | '\f' 
// handle newlines | ( options {generateAmbigWarnings=false;} : "\r\n" 
// Evil DOS 
| '\r' 
// Macintosh 
| '\n' 
// Unix (the right way) ) 
{ newline(); } )+ { _ttype = Token.SKIP; } 
; 

// a dummy rule to force vocabulary to be all characters (except special 
// ones that ANTLR uses internally (0 to 2) 
protected VOCAB : '\3'..'\377' ; 

// Single-line comments 
SL_COMMENT : "--" (~('\n'|'\r'))* ('\n'|'\r'('\n')?)? 
 	{$setType(Token.SKIP); newline();} 
; 
     

ARROW: "->" ;
BAR: '|';
COLON: ':' ;
COMMA: ',' ;
DIVIDE: '/' ;
DOT: '.' ;
DOTDOT: "..";
DOUBLECOLON: "::" ;
EQUALS:'=' ;
GE: ">=" ;
GT: '>';
LE: "<=" ;
LT: '<';
MINUS: '-' ;
MULTIPLY: '*' ;
NEQUALS: "<>" ;
LPAREN: '(' ;
RPAREN: ')' ;
LBRACKET: '{';
RBRACKET: '}';
PRE: "@pre" ;
PLUS: '+' ;
SEMICOLON: ';' ;


// a numeric literal
protected NUMBER_LITERAL:
    (DIGIT)+
;

protected REAL_LITERAL:
	(NUMBER_LITERAL DOT NUMBER_LITERAL)
	(EXPONENT) ?
;

protected EXPONENT : 
	('e'|'E') ('+'|'-')? ('0'..'9')+
; 


protected OPERATION_EXPR:
	(NUMBER_LITERAL DOT NAME)
;

// 1.34.56   - - numeric url name
protected NUMERIC_URL :
	NUMBER_LITERAL ( DOT (NUMBER_LITERAL | NAME))+
;

// a URL as part of a pathname   1.34.56::a::b::c
protected NUMERICPATHNAME : 
	NUMERIC_URL ( DOUBLECOLON (NAME|NUMBER_LITERAL) )+
;


NUMBER_OR_REAL_OR_RANGE_OR_OPERATION:
    ( NUMERIC_URL DOUBLECOLON ) => NUMERICPATHNAME {$setType(NUMERICPATHNAME); }
  |
  	( NUMBER_LITERAL DOT NAME) => OPERATION_EXPR { $setType(OPERATION_EXPR); }
  |
	( NUMBER_LITERAL DOT NUMBER_LITERAL)  => REAL_LITERAL {$setType(REAL_LITERAL);}
  | 
  	( NUMBER_LITERAL )  => NUMBER_LITERAL {$setType(NUMBER_LITERAL);}
;  
  	

// decimal digit
protected DIGIT: '0'..'9' ;


STRING_LITERAL: 
	'\'' 
	(ESC|~('\''|'\\'))* 
	'\'' 
;
// escape sequence -- note that this is protected; it can only be called 
// from another lexer rule -- it will not ever directly return a token to 
// the parser 
// There are various ambiguities hushed in this rule. The optional 
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched. ANTLR does the 
// right thing by matching immediately; hence, it's ok to shut off 
// the FOLLOW ambig warnings. 
protected ESC : '\\' ( 'n' | 'r' | 't' | 'b' | 'f' | '"' | '\'' | '\\' | ('u')+ 
HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
| '0'..'3' 
( options { warnWhenFollowAmbig = false; } : '0'..'7' 
( options { warnWhenFollowAmbig = false; } : '0'..'7' )? )? | '4'..'7' 
( options { warnWhenFollowAmbig = false; } : '0'..'7' )? ) ; 
// hexadecimal digit (again, note it's protected!) 
protected HEX_DIGIT : ('0'..'9'|'A'..'F'|'a'..'f') ; 

// variable name
protected NAME:
	('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')* 
;

// a variable name can have one or more embedded space
//protected NAME:
//	NAME1 ( ' ' NAME1)*
//;

protected NAMEORDOTTEDNAME:
	NAME ( DOT NAME )*
;

protected PACKAGE_NAME:
	NAME (' ' NAME)*
;

// A path name is:   a::b::c
//					www.what.ever::a::b::c
protected PATHNAME:
	NAMEORDOTTEDNAME ( DOUBLECOLON ( PACKAGE_NAME | NUMBER_LITERAL ) )+ 
;

NAME_OR_PATHNAME:
//		(NAME DOUBLECOLON) => PATHNAME {$setType(PATHNAME);}
		(NAMEORDOTTEDNAME DOUBLECOLON) => PATHNAME {$setType(PATHNAME);}
	|
		( NAME ) => NAME {$setType(NAME); }
;
	

