/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2005. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.menus.popup.actions;

import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.uml2.Element;

import com.ibm.xtools.emf.msl.ResourceSetReadOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.menus.MenusPlugin;
import com.ibm.xtools.notation.View;

public class SampleObjectPopupAction
	implements IObjectActionDelegate {

	private static final String KEY_MESSAGE_COMMON = "objectPopup.message.common"; //$NON-NLS-1$

	private static final String KEY_MESSAGE_NULL = "objectPopup.message.null"; //$NON-NLS-1$

	private static final String KEY_MESSAGE_UML = "objectPopup.message.uml"; //$NON-NLS-1$

	private static final String KEY_MESSAGE_NOTATION = "objectPopup.message.notation"; //$NON-NLS-1$

	private static final String KEY_MESSAGE_EOBJECT = "objectPopup.message.eobject"; //$NON-NLS-1$

	private static final String KEY_MESSAGE_INTERRUPTED = "objectPopup.message.interrupted"; //$NON-NLS-1$

	private static final String KEY_MESSAGE_EXCEPTION = "objectPopup.message.exception"; //$NON-NLS-1$

	private static final String KEY_TITLE = "title"; //$NON-NLS-1$

	private Shell shell = null;

	/**
	 * Constructor for Action.
	 */
	public SampleObjectPopupAction() {
		super();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {

		try {
			UMLModeler.getEditingDomain().run(new ResourceSetReadOperation() {

				protected void execute(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
				
					String message = MenusPlugin
						.getResourceString(KEY_MESSAGE_COMMON);

					EObject eObject = getSelectedEObject();
					
					if (eObject == null) {

						message += MenusPlugin
							.getResourceString(KEY_MESSAGE_NULL);

					} else if (eObject instanceof Element) {

						Object args[] = new Object[] {
							eObject.eClass().getName(),
							UMLModeler.getEObjectHelper().getQName(eObject,
								true)};

						message += MessageFormat.format(MenusPlugin
							.getResourceString(KEY_MESSAGE_UML), args);

					} else if (eObject instanceof View) {

						Object args[] = new Object[] {
							eObject.eClass().getName(),
							UMLModeler.getEObjectHelper().getQName(eObject,
								true)};

						message += MessageFormat.format(MenusPlugin
							.getResourceString(KEY_MESSAGE_NOTATION), args);

					} else {

						Object args[] = new Object[] {
							eObject.eClass().getName(), eObject.toString()};

						message += MessageFormat.format(MenusPlugin
							.getResourceString(KEY_MESSAGE_EOBJECT), args);

					}

					MessageDialog.openInformation(shell, MenusPlugin
						.getResourceString(KEY_TITLE), message);
				}
			}, new NullProgressMonitor());

		} catch (InvocationTargetException e) {

			MessageDialog.openInformation(shell, MenusPlugin
				.getResourceString(KEY_TITLE), MenusPlugin
				.getResourceString(KEY_MESSAGE_EXCEPTION));
			throw new RuntimeException(e);

		} catch (InterruptedException e) {

			MessageDialog.openInformation(shell, MenusPlugin
				.getResourceString(KEY_TITLE), MenusPlugin
				.getResourceString(KEY_MESSAGE_INTERRUPTED));

		}

	}
	
	/**
	 * This method returns the currently selected object if it is an EObject
	 * otherwise, just returns null.
	 */
	private EObject getSelectedEObject() {
		// Get selection.
		List elements = UMLModeler.getUMLUIHelper().getSelectedElements();
		// we're only interested in the first element.
		if ( elements.size() > 0) {
			Object object = elements.get(0);
			if (object instanceof EObject) {
				return (EObject)object;
			}
		}	
		return null;
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, final ISelection selection) {
		// no-op
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
	 *      org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

}