/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.modification.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.uml2.Property;
import org.eclipse.uml2.UML2Package;

import com.ibm.xtools.emf.msl.ResourceSetModifyOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.modification.ModificationPlugin;
import com.ibm.xtools.notation.View;

/**
 * This action delegate walks the selected elements and adds a property to each
 * visited class.
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public class AddProperty
	implements IWorkbenchWindowActionDelegate {

	/**
	 * Key for message to output when the operation was cancelled
	 */
	private static final String KEY_OPERATION_LABEL = "operationLabel"; //$NON-NLS-1$

	/**
	 * Key for message to output when the operation was cancelled
	 */
	private static final String KEY_PROPERTY_NAME_PREFIX = "propertyNamePrefix"; //$NON-NLS-1$

	/**
	 * Key for title of message dialog
	 */
	private static final String KEY_TITLE = "title"; //$NON-NLS-1$

	/**
	 * Key for message to output
	 */
	private static final String KEY_MESSAGE = "message"; //$NON-NLS-1$

	/**
	 * The shell this action delegate was initialized for
	 */
	private Shell shell;

	/**
	 * Constructor
	 */
	public AddProperty() {
		//No-op
	}

	/**
	 * Walk the selected objects and log them to the console
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		// Get selection
		final List elements = UMLModeler.getUMLUIHelper().getSelectedElements();

		final boolean performedOperation[] = {false};

		try {
			UMLModeler.getEditingDomain().run(
				new ResourceSetModifyOperation(ModificationPlugin
					.getResourceString(KEY_OPERATION_LABEL)) {

					protected void execute(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {

						// For each selected element
						for (Iterator iter = elements.iterator(); iter
							.hasNext();) {
							Object object = iter.next();

							// If a view, try to get semantic bridge
							if (object instanceof View) {
								object = ((View) object).getElement();
							}

							// If element is a UML Class
							if (object instanceof org.eclipse.uml2.Class) {

								org.eclipse.uml2.Class clazz = (org.eclipse.uml2.Class) object;

								// Find next unused property name
								int seqNo = 1;
								String prefix = ModificationPlugin
									.getResourceString(KEY_PROPERTY_NAME_PREFIX);

								String name;

								do {
									name = prefix + String.valueOf(seqNo++);
								} while (null != clazz.getOwnedAttribute(name));

								// Create property
								Property property = clazz
									.createOwnedAttribute(UML2Package.eINSTANCE
										.getProperty());
								property.setName(name);
								
								performedOperation[0] = true;
							}
						}

					}
				}, new NullProgressMonitor());

		} catch (InvocationTargetException e) {
			throw new RuntimeException(e.getCause());
		} catch (InterruptedException e) {
			// Interrupted by user
		}

		if (false == performedOperation[0]) {

			MessageDialog.openInformation(shell, ModificationPlugin
				.getResourceString(KEY_TITLE), ModificationPlugin
				.getResourceString(KEY_MESSAGE));
		}

	}

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow windowIn) {
		shell = windowIn.getShell();
	}
}