/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltomodel.classtoservice.multiplerules.transforms;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.uml2.UML2Package;

import com.ibm.xtools.transform.core.ITransformationDescriptor;
import com.ibm.xtools.transform.samples.modeltomodel.classtoservice.rules.CreateFactoryClassRule;
import com.ibm.xtools.transform.samples.modeltomodel.classtoservice.rules.CreateRealizationRule;
import com.ibm.xtools.transform.samples.modeltomodel.classtoservice.rules.SetupTargetRule;
import com.ibm.xtools.transform.uml2.UMLKindTransform;

/**
 * This transformation accepts a class as the source and creates the following:
 * <ul>
 * <li>An interface containing the public operations of the source class and
 * the accessor methods for the attributes</li>
 * <li>An implementation class that is a copy of the source class plus the
 * accessor methods</li>
 * <li>A factory class containing an operation that creates an instance of the
 * implementation class.
 * </ul>
 * 
 * This implementation of the transformation uses multiple rules and
 * sub-transforms to carry out the necessary conversions from the source class
 * to the target classes.
 *  
 */
public class ClassToServiceMultpleRuleTransform
	extends UMLKindTransform {

	public final static String ID = "com.ibm.xtools.transform.samples.modeltomodel.class2service"; //$NON-NLS-1$

	/**
	 * Constructor.
	 * 
	 * @param descriptor A transformation descriptor
	 */
	public ClassToServiceMultpleRuleTransform(ITransformationDescriptor descriptor) {
		super(descriptor);
		initializeTransform();
	}

	/**
	 * Construtor.
	 * 
	 * @param id An unique transformation id.
	 */
	public ClassToServiceMultpleRuleTransform(String id) {
		super(id);
		initializeTransform();
	}

	/**
	 * Sets up the contained transform elements (rules, extractors, and
	 * sub-transforms for this transform.
	 *  
	 */
	private void initializeTransform() {

		EClass classKind = UML2Package.eINSTANCE.getClass_();

		addByKind(classKind, new SetupTargetRule());
		addByKind(classKind, new ClassToInterfaceTransform(
			ClassToInterfaceTransform.ID));
		addByKind(classKind, new ClassToImplementationTransform(
			ClassToImplementationTransform.ID));

		addByKind(classKind, new CreateRealizationRule());
		addByKind(classKind, new CreateFactoryClassRule());

	}

}