/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltotext.classtotext.console.transforms;

import org.eclipse.uml2.UML2Package;

import com.ibm.xtools.transform.core.ITransformationDescriptor;
import com.ibm.xtools.transform.samples.modeltotext.classtotext.console.rules.ClassRule;
import com.ibm.xtools.transform.samples.modeltotext.classtotext.console.rules.OperationRule;
import com.ibm.xtools.transform.samples.modeltotext.classtotext.console.rules.ParameterRule;
import com.ibm.xtools.transform.samples.modeltotext.classtotext.console.rules.PropertyRule;
import com.ibm.xtools.transform.uml2.UMLKindTransform;

/**
 * Accepts a class and outputs its name and the names of its attributes,
 * operations and parameters to the console.
 */
public class ClassToTextConsoleTransform
	extends UMLKindTransform {

	public static final String ID = "com.ibm.xtools.transform.samples.modeltotext.classtotext.console"; //$NON-NLS-1$

	/**
	 * Constructor.
	 * 
	 * @param descriptor A transformation descriptor
	 */
	public ClassToTextConsoleTransform(ITransformationDescriptor descriptor) {
		super(descriptor);
		initializeTransform();
	}

	/**
	 * Constructor
	 * 
	 * @param id Unique id of the transform.
	 */
	public ClassToTextConsoleTransform(String id) {
		super(id);
		initializeTransform();
	}

	/**
	 * Sets up the rules used in this transform.
	 *  
	 */
	private void initializeTransform() {
		UML2Package uml2 = UML2Package.eINSTANCE;

		addByKind(uml2.getClass_(), new ClassRule());
		addByKind(uml2.getProperty(), new PropertyRule());
		addByKind(uml2.getOperation(), new OperationRule());
		addByKind(uml2.getParameter(), new ParameterRule());
	}

}