/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltotext.classtotext.file.rules;

import java.io.StringWriter;

import org.eclipse.uml2.NamedElement;
import org.eclipse.uml2.UML2Package;

import com.ibm.xtools.transform.core.AbstractRule;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.transform.samples.modeltotext.l10n.ResourceManager;
import com.ibm.xtools.transform.uml2.IsElementKindCondition;

/**
 * A rule to output the name of an operation in a class.
 */
public class OperationRule extends AbstractRule {
    
    public static final String ID = "classtotext.file.operation.rule"; //$NON-NLS-1$
    public static final String NAME = ResourceManager.getString("ClassToText.operationRule.name"); //$NON-NLS-1$
    private static final String EOL = System.getProperty("line.separator"); //$NON-NLS-1$
    /**
     * Default constructor.
     * 
     * Initializes the id and the name of the rule to
     * their default values. Also, initializes the rule to accept a UML
     * operation as source.
     */
    public OperationRule() {
    	super(ID, NAME);
        setAcceptCondition(new IsElementKindCondition(UML2Package.eINSTANCE.getOperation()));
    }
    
    /**
     * Creates the rule and initializes its id and the name. Also, initializes
     * the rule to accept a UML operation as source.
     * 
     * @param id Unique id of the rule
     * @param name Readable name of the rule
     */
    public OperationRule(String id, String name) {
        super(id, name);
        setAcceptCondition(new IsElementKindCondition(UML2Package.eINSTANCE.getOperation()));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.xtools.transform.uml2.internal.AbstractUMLRule#doCreate(com.ibm.xtools.transform.core.internal.engine.ITransformContext)
     */
    protected Object createTarget(ITransformContext context) {
    	NamedElement element = (NamedElement) context.getSource();
    	Object targetWriter = context.getPropertyValue("targetWriter");//$NON-NLS-1$
    	if (targetWriter instanceof StringWriter) {
    		((StringWriter)targetWriter).write("\tOperation: " + element.getName() + EOL); //$NON-NLS-1$
    	}
        return null;
    }

}
