/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.xtools.transform.samples.modeltotext.statetojsp.rules;

import java.util.HashMap;

import org.eclipse.uml2.NamedElement;
import org.eclipse.uml2.UML2Package;

import com.ibm.xtools.transform.core.AbstractRule;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.transform.samples.modeltotext.JavaNameTool;
import com.ibm.xtools.transform.samples.modeltotext.jet.DisplayTemplate;
import com.ibm.xtools.transform.samples.modeltotext.jet.EditFormTemplate;
import com.ibm.xtools.transform.samples.modeltotext.jet.FormTemplate;
import com.ibm.xtools.transform.samples.modeltotext.jet.SelectListTemplate;
import com.ibm.xtools.transform.samples.modeltotext.l10n.ResourceManager;
import com.ibm.xtools.transform.uml2.IsElementKindCondition;

/**
 * A rule that accepts a UML state marked up with stereotypes from the
 * ScreenFlowProfile or equivalent keyword and creates a JSP page.
 */
public class StateRule
	extends AbstractRule {

	public final static String ID = "com.ibm.xtools.transform.samples.state2jsp.state.rule"; //$NON-NLS-1$
	public final static String NAME = ResourceManager.getString("StateToJsp.stateRule.name"); //$NON-NLS-1$

	/**
	 * Default constructor. Initializes the name and the id of the rule, and
	 * initializes itself to accept only a UML state as source.
	 */
	public StateRule() {
		super(ID, NAME);
		setAcceptCondition(new IsElementKindCondition(UML2Package.eINSTANCE.getState()));
	}
	
	/**
	 * @param id Unique id of the rule.
	 * @param name Readable name of the rule
	 */
	public StateRule(String id, String name) {
		super(id, name);
		setAcceptCondition(new IsElementKindCondition(UML2Package.eINSTANCE.getState()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.xtools.transform.uml2.internal.AbstractUMLRule#doCreate(com.ibm.xtools.transform.core.internal.engine.ITransformContext)
	 */
	protected Object createTarget(ITransformContext context) {
		NamedElement element = (NamedElement) context.getSource();

		// Populate a newly created context, execute the appropriate
		// template and save the template output as a jsp file.
		HashMap templateContext = new HashMap();
		templateContext.put("JavaNameTool", new JavaNameTool()); //$NON-NLS-1$
		templateContext.put("state", element); //$NON-NLS-1$

		String textOutput = null;

		// Limitation: If the model element has more than one stereotype 
		// from the ScreenFlow profile, only the first stereotype is used.
		// This limitation arises from the one-to-one mapping between the
		// stereotype and the template.

		if (element.getAppliedStereotype("ScreenFlow::display") != null) { //$NON-NLS-1$
			textOutput = new DisplayTemplate().generate(templateContext);
		} else if (element.getAppliedStereotype("ScreenFlow::editform") != null) { //$NON-NLS-1$
			textOutput = new EditFormTemplate().generate(templateContext);
		} else if (element.getAppliedStereotype("ScreenFlow::form") != null) { //$NON-NLS-1$
			textOutput = new FormTemplate().generate(templateContext);
		} else if (element.getAppliedStereotype("ScreenFlow::selectlist") != null) { //$NON-NLS-1$
			textOutput = new SelectListTemplate().generate(templateContext);
		}
		
		context.setPropertyValue("outputBuffer", textOutput); //$NON-NLS-1$

		return null;
	}
}